//+------------------------------------------------------------------+
//|                                                           mladen |
//+------------------------------------------------------------------+
#property  indicator_separate_window
#property  indicator_buffers 3
#property  indicator_color1  Gray
#property  indicator_color2  DarkGray
#property  indicator_color3  LimeGreen

//
//
//
//
//

extern int    FastEMA            =12;
extern int    SlowEMA            =26;
extern int    SignalSMA          = 9;
extern int    PriceType          = 0;
extern string timeFrame          ="H1";
extern bool   alertsOn           = false;
extern bool   alertsOnCurrent    = false;
extern bool   alertsMessage      = true;
extern bool   alertsSound        = false;
extern bool   alertsEmail        = false;
extern bool   alertsNotification     = false;
extern bool   verticalLinesVisible   = true;
extern color  verticalLinesUpColor   = DeepSkyBlue;
extern color  verticalLinesDownColor = PaleVioletRed;
extern int    verticalLinesStyle     = STYLE_DOT;
extern int    verticalLinesWidth     = 0;
extern string verticalLinesID        = "MACD_Line";

//
//
//
//

double     SignalBuffer[];
double     MacdBuffer[];
double     OutlBuffer[];
double     trend[];

//
//
//
//
//

datetime TimeArray[];
int      TimeFrame;
int      correction;


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+

int init()
{
   IndicatorBuffers(4);
   SetIndexBuffer(0,MacdBuffer);
   SetIndexBuffer(1,OutlBuffer);
   SetIndexBuffer(2,SignalBuffer);
   SetIndexBuffer(3,trend);
   SetIndexLabel(0,"MACD");
   SetIndexLabel(1,"MACD");
   SetIndexLabel(2,"Signal");
   SetIndexStyle(0,DRAW_HISTOGRAM);
   
   //
   //
   //
   //
   //
   
   TimeFrame=stringToTimeFrame(timeFrame);
      IndicatorDigits(Digits+1);
      IndicatorShortName("mtf.MACD ("+TimeFrameToString(TimeFrame)+","+PriceTypeToString(PriceType)+") ("+FastEMA+","+SlowEMA+","+SignalSMA+")");
   
   //
   //
   //
   //
   //

   correction = MathMax(FastEMA,MathMax(SlowEMA,SignalSMA));
   SignalSMA = SignalSMA*TimeFrame/Period();
      for (int i=0;i<indicator_buffers;i++) SetIndexDrawBegin(i,correction);
   return(0);
}

//
//
//
//
//

int deinit() {
   string lookFor       = verticalLinesID+":";
   int    lookForLength = StringLen(lookFor);
   for (int i=ObjectsTotal()-1; i>=0; i--) {
      string objectName = ObjectName(i);
         if (StringSubstr(objectName,0,lookForLength) == lookFor) ObjectDelete(objectName);
   }
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+

int start()
{
   int      counted_bars=IndicatorCounted();
   int      limit,i,y=0;


   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
   limit=Bars-counted_bars;

      ArrayCopySeries(TimeArray ,MODE_TIME ,NULL,TimeFrame);
      
   //
   //
   //
   //
   //
   
      for(i=0;i<limit;i++)
         {
            if (Time[i]<TimeArray[y]) y++;
               MacdBuffer[i] = miMa(i,FastEMA,y)-miMa(i,SlowEMA,y);
               OutlBuffer[i] = MacdBuffer[i];
         }         
      for(i=limit; i>=0; i--)
      {
         SignalBuffer[i]=iMAOnArray(MacdBuffer,Bars,SignalSMA,0,MODE_SMA,i);
         trend[i] = trend[i+1];
            if (MacdBuffer[i]>SignalBuffer[i]) trend[i] =  1;
            if (MacdBuffer[i]<SignalBuffer[i]) trend[i] = -1;
            manageLines(i);
      }
      manageAlerts();            
      
   //
   //
   //
   //
   //
         
   return(0);
}


void manageAlerts()
{
   if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1; 

      //
      //
      //
      //
      //
            
      static datetime time1 = 0;
      static string   mess1 = "";
      if (trend[whichBar] != trend[whichBar+1])
      {
         if (trend[whichBar] ==  1) doAlert(time1,mess1,whichBar,"up");
         if (trend[whichBar] == -1) doAlert(time1,mess1,whichBar,"down");
      }
   }
}

//
//
//
//
//

void doAlert(datetime& previousTime, string& previousAlert, int forBar, string doWhat)
{
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

        message = TimeFrameToString(TimeFrame)+" "+Symbol()+" at "+TimeToStr(TimeLocal(),TIME_SECONDS)+" Macd crossed signal line "+doWhat;
          if (alertsMessage)      Alert(message);
          if (alertsEmail)        SendMail(StringConcatenate(Symbol()," macd"),message);
          if (alertsNotification) SendNotification(message);
          if (alertsSound)        PlaySound("alert2.wav");
   }
}


//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//
//
//
//
//

double miMa(int i,int maPeriod,int shift)
{
   double previous = 0.0;
   double pr       = 2.0 / (maPeriod+1);
   double price;
   


   previous = iMA(NULL,TimeFrame,maPeriod,0,MODE_EMA,PriceType,shift+1);

   //
   //
   //
   //
   //
   
   switch (PriceType)
      {
         case 0: price = Close[i];    break;
         case 1: price = Open[shift]; break;
         default :
            
            //
            //
            //
            //
            //
            
            for(int t=i;t>Bars;t++) if (Time[t] < TimeArray[shift]) break;
               double high = High[iHighest(NULL,0,MODE_HIGH,t-i+1,i)];
               double low  =   Low[iLowest(NULL,0,MODE_LOW ,t-i+1,i)];

            //
            //
            //
            //
            //
                           
            switch (PriceType)
               {
                  case 2: price = high;                           break;
                  case 3: price = low;                            break;
                  case 4: price = (high+low)/2;                   break;
                  case 5: price = (high+low+Close[i])/3;          break;
                  case 6: price = (high+low+Close[i]+Close[i])/4; break;
               }                     
      }         
   return(price*pr+previous*(1-pr));
}


//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//
//
//
//
//

string PriceTypeToString(int pt)
{
   string answer;
   switch(pt)
   {
      case 0:  answer = "Close"    ; break; 
      case 1:  answer = "Open"     ; break;
      case 2:  answer = "High"     ; break;
      case 3:  answer = "Low"      ; break;
      case 4:  answer = "Median"   ; break;
      case 5:  answer = "Typical"  ; break;
      case 6:  answer = "Wighted"  ; break;
      default: answer = "Invalid price field requested";
                                    Alert(answer);
   }
   return(answer);
}
int stringToTimeFrame(string tfs)
{
   int tf=0;
       tfs = StringUpperCase(tfs);
         if (tfs=="M1" || tfs=="1")     tf=PERIOD_M1;
         if (tfs=="M5" || tfs=="5")     tf=PERIOD_M5;
         if (tfs=="M15"|| tfs=="15")    tf=PERIOD_M15;
         if (tfs=="M30"|| tfs=="30")    tf=PERIOD_M30;
         if (tfs=="H1" || tfs=="60")    tf=PERIOD_H1;
         if (tfs=="H4" || tfs=="240")   tf=PERIOD_H4;
         if (tfs=="D1" || tfs=="1440")  tf=PERIOD_D1;
         if (tfs=="W1" || tfs=="10080") tf=PERIOD_W1;
         if (tfs=="MN" || tfs=="43200") tf=PERIOD_MN1;
         if (tf<Period()) tf=Period();
  return(tf);
}
string TimeFrameToString(int tf)
{
   string tfs="Current time frame";
   switch(tf) {
      case PERIOD_M1:  tfs="M1"  ; break;
      case PERIOD_M5:  tfs="M5"  ; break;
      case PERIOD_M15: tfs="M15" ; break;
      case PERIOD_M30: tfs="M30" ; break;
      case PERIOD_H1:  tfs="H1"  ; break;
      case PERIOD_H4:  tfs="H4"  ; break;
      case PERIOD_D1:  tfs="D1"  ; break;
      case PERIOD_W1:  tfs="W1"  ; break;
      case PERIOD_MN1: tfs="MN1";
   }
   return(tfs);
}

//
//
//
//
//

string StringUpperCase(string str)
{
   string   s = str;
   int      tchar;
   
   for(int i=StringLen(str)-1; i>=0; i--)
      {
         tchar = StringGetChar(s, i);
         
         //
         //
         //
         //
         //
         
         if((tchar > 96 && tchar < 123) || (tchar > 223 && tchar < 256))
                  s = StringSetChar(s, i, tchar - 32);
          else 
              if(tchar > -33 && tchar < 0)
                  s = StringSetChar(s, i, tchar + 224);
                  
   }
   
   //
   //
   //
   //
   //
   
   return(s);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------

void manageLines(int i) {
   if (verticalLinesVisible) 
   {
         deleteLine(Time[i]);
         if (trend[i]!=trend[i+1]) 
         {
            if (trend[i] == 1 ) drawLine(i,verticalLinesUpColor);
            if (trend[i] ==-1 ) drawLine(i,verticalLinesDownColor);
         }
   }
}               
void drawLine(int i,color theColor) {
   string name = verticalLinesID+":"+Time[i];
   
      ObjectCreate(name,OBJ_VLINE,0,Time[i],0);
         ObjectSet(name,OBJPROP_COLOR,theColor);
         ObjectSet(name,OBJPROP_STYLE,verticalLinesStyle);
         ObjectSet(name,OBJPROP_WIDTH,verticalLinesWidth);
         ObjectSet(name,OBJPROP_BACK,true);
}
void deleteLine(datetime time) {
   string lookFor = verticalLinesID+":"+time; ObjectDelete(lookFor);
}